4206
%{
#include "StdH.h"
#include "Player.h"
#include "PlayerWeapons.h"
#include "Light.h"
%}

class CFlashlight: CRationalEntity {
name      "Flashlight";
thumbnail "Thumbnails\\Light.tbn";
features  "HasName", "IsTargetable";

properties:

  1 CTString m_strName                     "Name" 'N' = "Flashlight",         // class name
  2 CEntityPointer m_penWeapons,
  3 CEntityPointer m_penPlayer,
  4 CEntityPointer m_penLight              "Light",
  6 COLOR m_colNear			"Color near" =C_GRAY,
  7 COLOR m_colFar			"Color far" =C_dGRAY,
  8 RANGE m_rNear			"Range near" =8.0f,
  9 RANGE m_rFar			"Range far"  =20.0f,
 10 FLOAT m_fFallOffNear    "Fall off near" =4.0f,
 11 FLOAT m_fFallOffFar     "Fall off far" =10.0f,
 12 FLOAT m_fHotSpotNear    "Hot spot near" =1.0f,
 13 FLOAT m_fHotSpotFar     "Hot spot far"  =4.0f,

components:

  1 model   MODEL_MARKER     "Models\\Editor\\Lamp.mdl",
  2 texture TEXTURE_MARKER   "Models\\Editor\\SpotLight.tex"


functions:

  BOOL IsTargetValid(SLONG slPropertyOffset, CEntity *penTarget)
  {
    if( slPropertyOffset == offsetof(CFlashlight, m_penLight))
    {
      if (IsOfClass(penTarget, "Light")) { return TRUE; }
      else { return FALSE; }
    }   
    return CEntity::IsTargetValid(slPropertyOffset, penTarget);
  }   	                               

    // Adjust model mip factor if needed. 
   void AdjustMipFactor(FLOAT &fMipFactor)
   {
     fMipFactor = 0;
   }

  // returns bytes of memory used by this object
  SLONG GetUsedMemory(void)
  {
    // initial
    SLONG slUsedMemory = sizeof(CFlashlight) - sizeof(CRationalEntity) + CRationalEntity::GetUsedMemory();
    // add some more
    slUsedMemory += m_strName.Length();
    return slUsedMemory;
  }

  void GetPlayerWeapons(void) {
	{FOREACHINDYNAMICCONTAINER(GetWorld()->wo_cenEntities, CEntity, iten) {
		CEntity *pen = iten;
		if(pen) {
			if(IsOfClass(pen,"Player Weapons")) {
				m_penWeapons=pen;
			}
			if(IsOfClass(pen,"Player")) {
				m_penPlayer=pen;
			}
		}
	}}
  }

  void DeactivateRest(void) {
	{FOREACHINDYNAMICCONTAINER(GetWorld()->wo_cenEntities, CEntity, iten) {
		CEntity *pen = iten;
		if(pen) {
			if(IsOfClass(pen,"Flashlight")&&pen!=this) {
				pen->SendEvent(EStop());
			}
		}
	}}
  }

void ReconsiderParameters(FLOAT fDistance) {
	if(m_rFar==m_rNear) { return; } 
	FLOAT fAmount=(fDistance-m_rNear)/(m_rFar-m_rNear);
	((CLight&)(*m_penLight)).m_rFallOffRange=m_fFallOffNear+(m_fFallOffFar-m_fFallOffNear)*fAmount;
	((CLight&)(*m_penLight)).m_rHotSpotRange=m_fHotSpotNear+(m_fHotSpotFar-m_fHotSpotNear)*fAmount;
	INDEX cNR,cNG,cNB,cNA,cFR,cFG,cFB,cFA,cR,cG,cB,cA;

	cNR= (m_colNear & (0xFF << 0)) >> 0;
	cNG= (m_colNear & (0xFF << 8)) >> 8;
	cNB= (m_colNear & (0xFF << 16)) >> 16;
	cNA= (m_colNear & (0xFF << 24)) >> 24;
	cFR= (m_colFar & (0xFF << 0)) >> 0;
	cFG= (m_colFar & (0xFF << 8)) >> 8;
	cFB= (m_colFar & (0xFF << 16)) >> 16;
	cFA= (m_colFar & (0xFF << 24)) >> 24;

	cR=cNR+(cFR-cNR)*fAmount;
	cG=cNG+(cFG-cNG)*fAmount;
	cB= cNB+(cFB-cNB)*fAmount;
	cA= cNA+(cFA-cNA)*fAmount;
	COLOR col = cR | cG<<8 | cB<<16| cA<<24;
	((CLight&)(*m_penLight)).m_colColor=col;
}

procedures:

Active() 
{
	while (TRUE)
	{
		wait (0.05f)
		{
			on (EBegin) : { resume; }
			on (EStop) : {
				if(m_penLight!=NULL) {
					((CLight&)(*m_penLight)).m_colColor=C_BLACK;
					EChangeAnim ean;
					ean.iLightAnim=((CLight&)(*m_penLight)).m_iLightAnimation;
					ean.iAmbientLightAnim=((CLight&)(*m_penLight)).m_iAmbientLightAnimation;
					ean.bLightLoop=TRUE;
					ean.bAmbientLightLoop=TRUE;
					ean.colAmbient=((CLight&)(*m_penLight)).m_colAmbient;
					ean.colDiffuse=((CLight&)(*m_penLight)).m_colColor;
					m_penLight->SendEvent(ean);
				}
				return;
			}
			on (ETimer) : {
				if ((m_penWeapons==NULL) || (m_penPlayer==NULL)) {
					GetPlayerWeapons();
				}
				if(m_penWeapons!=NULL&&m_penPlayer!=NULL&&m_penLight!=NULL) {
					FLOAT3D fHitTarget=((CPlayerWeapons&)(*m_penWeapons)).m_vRayHit;
					FLOAT fDistance=(fHitTarget-m_penPlayer->GetPlacement().pl_PositionVector).Length();
					m_penLight->SetPlacement(CPlacement3D(fHitTarget,ANGLE3D(0,0,0)));
					if(fDistance<=m_rNear) {
						((CLight&)(*m_penLight)).m_colColor=m_colNear;
						((CLight&)(*m_penLight)).m_rFallOffRange=m_fFallOffNear;
						((CLight&)(*m_penLight)).m_rHotSpotRange=m_fHotSpotNear;
					} else if(fDistance>=m_rFar) {
						((CLight&)(*m_penLight)).m_colColor=m_colFar;
						((CLight&)(*m_penLight)).m_rFallOffRange=m_fFallOffFar;
						((CLight&)(*m_penLight)).m_rHotSpotRange=m_fHotSpotFar;
					} else ReconsiderParameters(fDistance);
					EChangeAnim ean;
					ean.iLightAnim=((CLight&)(*m_penLight)).m_iLightAnimation;
					ean.iAmbientLightAnim=((CLight&)(*m_penLight)).m_iAmbientLightAnimation;
					ean.bLightLoop=TRUE;
					ean.bAmbientLightLoop=TRUE;
					ean.colAmbient=((CLight&)(*m_penLight)).m_colAmbient;
					ean.colDiffuse=((CLight&)(*m_penLight)).m_colColor;
					m_penLight->SendEvent(ean);
				}
				stop;
			}
			otherwise() : { resume; }
		}
	}

	return;
}

  Main() {
    InitAsEditorModel();
    SetPhysicsFlags(EPF_MODEL_IMMATERIAL);
    SetCollisionFlags(ECF_IMMATERIAL);

    // set appearance
    SetModel(MODEL_MARKER);
    SetModelMainTexture(TEXTURE_MARKER);

	if(m_rNear<0.0f) { m_rNear=0.0f; }
	if(m_rNear>m_rFar) { m_rFar=m_rNear; }
	if(m_fFallOffNear<0.0f) { m_fFallOffNear=0.0f; }
	if(m_fFallOffFar<0.0f) { m_fFallOffFar=0.0f; }
	if(m_fHotSpotNear<0.0f) { m_fHotSpotNear=0.0f; }
	if(m_fHotSpotFar<0.0f) { m_fHotSpotFar=0.0f; }

    // spawn in world editor
    autowait(0.1f);
	
		wait() {
			on (EBegin) : { 
				if(m_penWeapons==NULL||m_penPlayer==NULL) {
					GetPlayerWeapons();
				}
				resume;
			}
			on (EStart) : {
				DeactivateRest();
				call Active();
			}			
			otherwise() : { resume; }
		}

    // cease to exist
    Destroy();

    return;
  };
};
